const fetch = require('node-fetch');
const dotenv = require('dotenv').config();
const fs = require('fs')
const { CLIENT_ID, BH_CLIENT_SECRET, BULLHORN_USER, BULLHORN_PASSWORD, BASE_URL } = process.env

var getToken = async (req, res, next) => {
    return new Promise(function (resolve, reject) {
        fs.readFile(`${__dirname}/tokendata.json`, 'utf8', (err, jsonString) => {
            if (err) {
                reject(err)
                console.log('File read failed:')
            }
            resolve(JSON.parse(jsonString))
            // res.send(JSON.parse(jsonString))
        })
    })
}

var getCandidate = async (req, res, next) => {
    const { candidateId } = req.query
    const gettoken = await getToken()
    const fields = "id,address,email,name,dateAdded,willRelocate,travelLimit,preferredContact,occupation,owner,salary,customText10,customText1,status,phone"
    if (gettoken.BhRestToken) {
        fetch(`${BASE_URL}/entity/Candidate/${candidateId}?fields=${fields}&BhRestToken=${gettoken.BhRestToken}`)
            .then(async (result) => {
                var actualRes = await result.json()
                res.send(actualRes)
            })
            .catch(err => res.send(err));
    }
}

var getCandidateByEmail = async (req, res, next) => {
    const { email } = req.query
    const gettoken = await getToken()
    const fields = `id,address,email,name,dateAdded,willRelocate,travelLimit,preferredContact,occupation,owner,salary,customText10,customText1,status,phone&query=email:${email}`
    if (gettoken.BhRestToken) {
        fetch(`${BASE_URL}/search/Candidate?fields=${fields}&BhRestToken=${gettoken.BhRestToken}`)
            .then(async (result) => {
                var actualRes = await result.json()
                res.send(actualRes)
            })
            .catch(err => res.send(err));
    }
}

var getCandidateByPhone = async (req, res, next) => {
    const { phone } = req.query
    const gettoken = await getToken()
    const fields = `id,address,email,name,dateAdded,willRelocate,travelLimit,preferredContact,occupation,owner,customText10,customText1,salary,status,phone&query=phone:${phone}`
    if (gettoken.BhRestToken) {
        fetch(`${BASE_URL}/search/Candidate?fields=${fields}&BhRestToken=${gettoken.BhRestToken}`)
            .then(async (result) => {
                var actualRes = await result.json()
                res.send(actualRes)
            })
            .catch(err => res.send(err));
    }
}

var getTearSheet = async (req, res, next) => {
    const { candidateId } = req.query
    const gettoken = await getToken()
    if (gettoken.BhRestToken) {
        fetch(`${BASE_URL}/query/Tearsheet?BhRestToken=${gettoken.BhRestToken}&where=candidates.id%3D${candidateId}&fields=id,name,jobOrders`)
            .then(async result => {
                var actualRes = await result.json()
                res.send(actualRes)
            })
            .catch(err => res.send(err));
    }

}

var getJoborder = async (req, res, next) => {
    const { jobId } = req.query
    const gettoken = await getToken()
    const fields = "id,clientCorporation,title,owner,status,salary,address,categories,travelRequirements,customObject1s"
    if (gettoken.BhRestToken) {
        fetch(`${BASE_URL}/query/JobOrder?BhRestToken=${gettoken.BhRestToken}&where=id%3D${jobId}&fields=${fields}`)
            .then(async result => {
                var actualRes = await result.json()
                res.send(actualRes)
            })
            .catch(err => res.send(err));
    }

}

var JoborderCustomObject = async (req, res, next) => {
    const { customObjectId } = req.query
    const gettoken = await getToken()
    if (gettoken.BhRestToken) {
        fetch(`${BASE_URL}/query/JobOrderCustomObjectInstance1?BhRestToken=${gettoken.BhRestToken}&where=id%3D${customObjectId}&fields=*`)
            .then(async result => {
                var actualRes = await result.json()
                res.send(actualRes)
            })
            .catch(err => res.send(err));
    }
}

var getJobSubmission = async (req, res, next) => {
    const { jobSubmissionId } = req.query
    const gettoken = await getToken()
    const fields = "id,candidate,jobOrder,owners,sendingUser,customText1,customText2,customText3,customText4,customText5,customText6,customText7,customTextBlock1,customTextBlock2,customTextBlock3,customInt1,customInt2,customInt3,customInt4"
    if (gettoken.BhRestToken) {
        fetch(`${BASE_URL}/entity/JobSubmission/${jobSubmissionId}?BhRestToken=${gettoken.BhRestToken}&fields=${fields}`)
            .then(async result => {
                var actualRes = await result.json()
                res.send(actualRes)
            })
            .catch(err => res.send(err));
    }
}

var createJobSubmission = async (req, res, next) => {
    const { jobSubmissionId } = req.query
    const { data } =  req.body
    const gettoken = await getToken()
    if (gettoken.BhRestToken) {
        if(jobSubmissionId){
            fetch(`${BASE_URL}/entity/JobSubmission/${jobSubmissionId}?BhRestToken=${gettoken.BhRestToken}`,{
                method: 'POST', // *GET, POST, PUT, DELETE, etc.
                mode: 'cors', // no-cors, *cors, same-origin
                cache: 'no-cache', // *default, no-cache, reload, force-cache, only-if-cached
                credentials: 'same-origin', // include, *same-origin, omit
                headers: {
                  'Content-Type': 'application/json'
                  // 'Content-Type': 'application/x-www-form-urlencoded',
                },
                redirect: 'follow', // manual, *follow, error
                referrerPolicy: 'no-referrer', // no-referrer, *no-referrer-when-downgrade, origin, origin-when-cross-origin, same-origin, strict-origin, strict-origin-when-cross-origin, unsafe-url
                body: JSON.stringify(data) // body data type must match "Content-Type" header
              })
            .then(async result => {
                var actualRes = await result.json()
                res.send(actualRes)
            })
            .catch(err => res.send(err));
        }else{
            fetch(`${BASE_URL}/entity/JobSubmission?BhRestToken=${gettoken.BhRestToken}`,{
                method: 'PUT', // *GET, POST, PUT, DELETE, etc.
                mode: 'cors', // no-cors, *cors, same-origin
                cache: 'no-cache', // *default, no-cache, reload, force-cache, only-if-cached
                credentials: 'same-origin', // include, *same-origin, omit
                headers: {
                  'Content-Type': 'application/json'
                  // 'Content-Type': 'application/x-www-form-urlencoded',
                },
                redirect: 'follow', // manual, *follow, error
                referrerPolicy: 'no-referrer', // no-referrer, *no-referrer-when-downgrade, origin, origin-when-cross-origin, same-origin, strict-origin, strict-origin-when-cross-origin, unsafe-url
                body: JSON.stringify(data) // body data type must match "Content-Type" header
              })
                .then(async result => {
                    var actualRes = await result.json()
                    res.send(actualRes)
                })
                .catch(err => res.send(err));
        }
        
    }
}

var authorized = async () => {
    let url = `https://auth.bullhornstaffing.com/oauth/authorize?client_id=${CLIENT_ID}&response_type=code&action=Login&username=${BULLHORN_USER}&password=${BULLHORN_PASSWORD}`;

    let response = await fetch(url, {
        method: 'GET',
    });

    const redirectUrl = response.url;
    var arr = redirectUrl.split("code=");
    var authToken = arr[1];
    return authToken
}

var Oauth = async () => {
    var authToken = await authorized()

    let accessUrl = `https://auth.bullhornstaffing.com/oauth/token?grant_type=authorization_code&code=` + authToken + `&client_secret=${BH_CLIENT_SECRET}`;

    let accessResponse = await fetch(accessUrl, {
        method: 'POST',
    }).then(res => res.json())
        .then(async json => {
            const access_token = json;
            return access_token
        });
    return accessResponse
}

var getAccessToken = async (req, res, next) => {
    var data = await Oauth()
    var access_token = data.access_token

    fetch('https://rest.bullhornstaffing.com/rest-services/login?version=2.0&access_token=' + access_token)
        .then(async (result) => {
            var actualRes = await result.json()
            var jsonString = JSON.stringify(actualRes)
            // res.send({...actualRes,data});
            console.log(`${__dirname}/tokendata.json`)
            fs.writeFile(`${__dirname}/tokendata.json`, jsonString, err => {
                if (err) {
                    res.send('Error writing file')
                    console.log('Error writing file', err)
                } else {
                    res.send('Successfully wrote file')
                    console.log('Successfully wrote file')
                }
            })
        })
        .catch(err => console.log('authorize', err));
}

var refreshToken = async (req, res, next) => {
    var refresh_token = req.query.refresh_token

    fetch(`https://auth.bullhornstaffing.com/oauth/token?grant_type=refresh_token&refresh_token=${refresh_token}&client_id=${CLIENT_ID}&client_secret=${BH_CLIENT_SECRET}`, {
        method: 'post',
        headers: { 'Content-Type': 'application/json' },
    })
        .then(async (result) => {
            var actualRes = await result.json()
            res.send(actualRes);
        })
        .catch(err => console.log('authorize', err));
}

var login = async (req, res, next) => {
    var access_token = req.query.access_token
    fetch(`https://rest.bullhornstaffing.com/rest-services/login?version=2.0&access_token=${access_token}`)
        .then(res => res.json())
        .then(
            json => console.log(json)
        );
}

const exportFunction = {
    login,
    getToken,
    authorized,
    getJoborder,
    getTearSheet,
    getCandidate,
    refreshToken,
    getAccessToken,
    getCandidateByEmail,
    getCandidateByPhone,
    JoborderCustomObject,
    getJobSubmission,
    createJobSubmission
}

module.exports = exportFunction


